@@ -25,6 +25,13 @@ ReplProj4string = function(obj, value) {
 			stop(mess)
 		}
 	}
+        if (!is.na(is.projected(obj)) && !is.na(p4str)) {
+            p4s <- proj4string(obj)
+            if (p4s != p4str) {
+                mess <- paste("A new CRS was assigned to an object with an existing CRS:\n", p4s, "\nwithout reprojecting.\nFor reprojection, use function spTransform in package rgdal", sep="")
+                if (get_ReplCRS_warn()) warning(mess)
+            }
+        }
 	obj@proj4string = value;
 	obj
 }
@@ -57,18 +64,10 @@ setReplaceMethod("proj4string", c("Spatial", "CRS"), ReplProj4string)
 setMethod("is.projected", signature(obj = "Spatial"),
 	function(obj) {
 		p4str <- proj4string(obj)
-#ifdef R
-		if (is.na(p4str) || nchar(p4str) == 0) 
-#else
-#S	if (p4str == "NA")  # bloody S-Plus!
-#endif
+		if (is.na(p4str) || !nzchar(p4str)) 
 			return(as.logical(NA))
 		else {
-#ifdef R
-			res <- grep("longlat", p4str, fixed=TRUE)
-#else
-#S		res <- grep("longlat", p4str)
-#endif
+			res <- grep("longlat", p4str, fixed = TRUE)
 			if (length(res) == 0)
 				return(TRUE)
 			else
